/* 
 * Text Trigger plugin for HeXHub (http://sourceforge.net/projects/hexhub)
 * (C) Copyright 2007 Morten Larsen, bluebear dot dk at gmail dot com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <windows.h>
#include <commctrl.h>
#include <string>
#include "resource.h"
#include "trig.h"

void SaveTrigger(HWND hWndDlg) {
	HANDLE hFile;
	DWORD bytes;
	std::string filename;
	char *buf;
	UINT lt;
	size_t len;
	Trigger t;

	memset(t.description, 0, 256);
	memset(t.name, 0, 256);
	memset(t.usercommand, 0, 256);

	if (GetDlgItemTextA(hWndDlg, IDC_EDIT1, t.name, 256) == 0) {
		MessageBox(hWndDlg, L"Name field can't be empty!", L"User error", MB_OK);
		return;
	}

	if (GetDlgItemTextA(hWndDlg, IDC_EDIT2, t.description, 256) == 0) {
		MessageBox(hWndDlg, L"Description field can't be empty!", L"User error", MB_OK);
		return;
	}

	GetDlgItemTextA(hWndDlg, IDC_EDIT3, t.usercommand, 255);

	lt = static_cast<UINT>(SendMessage(GetDlgItem(hWndDlg, IDC_EDIT4), EM_GETLIMITTEXT, 0, 0));

	buf = new char [lt];
	if (buf == 0) {
		MessageBox(hWndDlg, L"Out of memory!", L"Error", MB_OK);
		return;
	}

	if (GetDlgItemTextA(hWndDlg, IDC_EDIT4, buf, lt) == 0) {
		MessageBox(hWndDlg, L"Trigger text field can't be empty!", L"User error", MB_OK);
		delete [] buf;
		return;
	}

	filename = t.name;
	filename += ".trig";
	if ((hFile = CreateFileA(filename.c_str(), GENERIC_WRITE, 0, 0, CREATE_ALWAYS, 0, 0)) == INVALID_HANDLE_VALUE) {
		MessageBox(hWndDlg, L"Could not create file!", L"Error", MB_OK);
		delete [] buf;
		return;
	}

	if (WriteFile(hFile, reinterpret_cast<LPCVOID>(&t), sizeof(Trigger), &bytes, 0) == FALSE) {
		MessageBox(hWndDlg, L"Error while writing file!", L"Error", MB_OK);
		delete [] buf;
		CloseHandle(hFile);
		return;		
	}

	if (bytes != sizeof(Trigger)) {
		MessageBox(hWndDlg, L"Error while writing file!", L"Error", MB_OK);
		delete [] buf;
		CloseHandle(hFile);
		return;		
	}

	len = strlen(buf);
	if (WriteFile(hFile, buf, static_cast<DWORD>(len), &bytes, 0) == FALSE) {
		MessageBox(hWndDlg, L"Error while writing file!", L"Error", MB_OK);
		delete [] buf;
		CloseHandle(hFile);
		return;		
	}

	if (bytes != len) {
		MessageBox(hWndDlg, L"Error while writing file!", L"Error", MB_OK);
		delete [] buf;
		CloseHandle(hFile);
		return;		
	}

	delete [] buf;
	CloseHandle(hFile);
	MessageBoxA(hWndDlg, "Saved successfully.", filename.c_str(), MB_OK);
}

void LoadTrigger(HWND hWndDlg) {
	HANDLE hFile;
	DWORD bytes;
	char buf[4096];
	std::string text;
	Trigger t;
	OPENFILENAMEA ofn;

	memset(&ofn, 0, sizeof(ofn));
	ofn.lStructSize = sizeof(ofn);
	ofn.hwndOwner = hWndDlg;
	ofn.lpstrFile = buf;
	ofn.lpstrFile[0] = 0;
	ofn.nMaxFile = sizeof(buf);
	ofn.lpstrFilter = "Trigger Files\0*.trig\0\0";
	ofn.nFilterIndex = 1;
	ofn.Flags = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST;

	if (GetOpenFileNameA(&ofn) == FALSE) return;

	hFile = CreateFileA(buf, GENERIC_READ, 0, 0, OPEN_EXISTING, 0, 0);
	if (hFile == INVALID_HANDLE_VALUE) {
		CloseHandle(hFile);
		return;
	}

	if (GetFileSize(hFile, 0) == 0) {
		CloseHandle(hFile);
		return;
	}

	if (ReadFile(hFile, &t, sizeof(Trigger), &bytes, 0) == FALSE) {
		CloseHandle(hFile);
		return;
	}

	if (bytes != sizeof(Trigger)) {
		CloseHandle(hFile);
		return;
	}

	while (ReadFile(hFile, buf, 4095, &bytes, 0) == TRUE) {
		if (bytes == 0) break;
		buf[bytes] = 0;
		text += buf;
	}
	CloseHandle(hFile);

	SetDlgItemTextA(hWndDlg, IDC_EDIT1, t.name);
	SetDlgItemTextA(hWndDlg, IDC_EDIT2, t.description);
	SetDlgItemTextA(hWndDlg, IDC_EDIT3, t.usercommand);
	SetDlgItemTextA(hWndDlg, IDC_EDIT4, text.c_str());	
}

LRESULT CALLBACK DlgProc(HWND hWndDlg, UINT Msg, WPARAM wParam, LPARAM lParam) {
	switch(Msg) {
		case WM_INITDIALOG:
			SendMessage(GetDlgItem(hWndDlg, IDC_EDIT1), EM_LIMITTEXT, static_cast<WPARAM>(255), 0);
			SendMessage(GetDlgItem(hWndDlg, IDC_EDIT2), EM_LIMITTEXT, static_cast<WPARAM>(255), 0);
			SendMessage(GetDlgItem(hWndDlg, IDC_EDIT3), EM_LIMITTEXT, static_cast<WPARAM>(255), 0);
			return TRUE;
		case WM_CLOSE:
			EndDialog(hWndDlg, 0);
			return TRUE;
		case WM_COMMAND:
			switch(wParam) {
				case IDC_BUTTON1: 
					LoadTrigger(hWndDlg);
					return TRUE;
				case IDC_BUTTON2: 
					SaveTrigger(hWndDlg);
					return TRUE;
			}
	}
	return FALSE;
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow) {
	INITCOMMONCONTROLSEX InitCtrlEx;
	InitCtrlEx.dwSize = sizeof(INITCOMMONCONTROLSEX);
	InitCtrlEx.dwICC  = ICC_PROGRESS_CLASS;
	InitCommonControlsEx(&InitCtrlEx);
	DialogBox(hInstance, MAKEINTRESOURCE(IDD_DIALOG1), 0, reinterpret_cast<DLGPROC>(DlgProc));
	return 0;
}