/* 
 * Text Trigger plugin for HeXHub (http://sourceforge.net/projects/hexhub)
 * (C) Copyright 2007 Morten Larsen, bluebear dot dk at gmail dot com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <windows.h>
#include <shlwapi.h>
#include <string>
#include "hexhub.h"
#include "..\MkTrig\trig.h"

void HEXHUBAPI onCmd(long cid, long uid, void *r, void *cp, bool b);

TriggerList *first;
TriggerList *last;

void LoadTrigger(std::string *filename) {
	HANDLE hFile;
	DWORD bytes;
	char buf[4096];
	TriggerList *cur = new TriggerList;
	if (cur == 0) return;

	hFile = CreateFileA(filename->c_str(), GENERIC_READ, 0, 0, OPEN_EXISTING, 0, 0);
	if (hFile == INVALID_HANDLE_VALUE) {
		delete cur;
		CloseHandle(hFile);
		return;
	}

	if (GetFileSize(hFile, 0) == 0) {
		delete cur;
		CloseHandle(hFile);
		return;
	}

	if (ReadFile(hFile, &cur->t, sizeof(Trigger), &bytes, 0) == FALSE) {
		delete cur;
		CloseHandle(hFile);
		return;
	}

	if (bytes != sizeof(Trigger)) {
		delete cur;
		CloseHandle(hFile);
		return;
	}

	while (ReadFile(hFile, buf, 4095, &bytes, 0) == TRUE) {
		if (bytes == 0) break;
		buf[bytes] = 0;
		cur->text += buf;
	}
	CloseHandle(hFile);

	if ((cur->triggerId = HexHubFunctions::RegisterCommand(cur->t.name, onCmd)) == -1) {
		delete cur;
		return;
	}

	cur->registeredHelpString = "!";
	cur->registeredHelpString += cur->t.name;
	cur->registeredHelpString += "\t\t\t-\t";
	cur->registeredHelpString += cur->t.description;

	if ((cur->helpstringId = HexHubFunctions::RegisterHelpString("EN", cur->registeredHelpString.c_str(), 0)) == -1) {
		HexHubFunctions::UnregisterCommand(cur->triggerId);
		delete cur;
		return;
	}

	if (cur->t.usercommand[0] != 0) {
		cur->registeredUserCommand = "$UserCommand 1 1 ";
		cur->registeredUserCommand += cur->t.usercommand;
		cur->registeredUserCommand += "$<%[mynick]> !";
		cur->registeredUserCommand += cur->t.name;
		cur->registeredUserCommand += "&#124;|";
		cur->usercommandId = HexHubFunctions::RegisterUserCommand(cur->registeredUserCommand.c_str());
		if (cur->usercommandId == -1) {
			HexHubFunctions::UnregisterHelpString(cur->helpstringId);
			HexHubFunctions::UnregisterCommand(cur->triggerId);
			delete cur;
			return;
		}
	} else cur->registeredUserCommand = -1;

	cur->next = 0;

	if (first == 0) {
		first = cur;
		last = cur;
	} else {
		last->next = cur;
		last = cur;
	}
}

void LoadTriggers(void) {
	first = 0;
	last = 0;
	std::string filename;
	WIN32_FIND_DATAA FindFileData;
	HANDLE hFind;
	BOOL b = TRUE;
	hFind = FindFirstFileA(".\\Plugins\\TxtTrigs\\*.trig", &FindFileData);
	if (hFind == INVALID_HANDLE_VALUE) return;

	filename = ".\\Plugins\\TxtTrigs\\";
	if (FindFileData.cFileName) {
		filename += FindFileData.cFileName;
	} else {
		filename += FindFileData.cAlternateFileName;
	}

	LoadTrigger(&filename);

	while (b) {
		b = FindNextFileA(hFind, &FindFileData);
			if (b == TRUE) {
				filename = ".\\Plugins\\TxtTrigs\\";
				if (FindFileData.cFileName) {
					filename += FindFileData.cFileName;
				} else {
					filename += FindFileData.cAlternateFileName;
				}
				LoadTrigger(&filename);
			}
	}
	FindClose(hFind);
}

const char * HEXHUBAPI InitPlugin(lpfnGetFunction HexGetFunction) {
	HexHubFunctions hhf;
	if (hhf.Init(HexGetFunction) == false) return 0;

	if (PathFileExists(L".\\Plugins\\TxtTrigs\\") == FALSE) if (CreateDirectory(L".\\Plugins\\TxtTrigs\\", 0) == FALSE) return 0;

	LoadTriggers();

	return "Text Triggers 0.1 by bluebear";
}

void HEXHUBAPI UnloadPlugin(void) {
	TriggerList *del, *save;
	del = first;
	while (del != 0) {
		save = del->next;
		HexHubFunctions::UnregisterCommand(del->triggerId);
		HexHubFunctions::UnregisterHelpString(del->helpstringId);
		if (del->usercommandId != -1) HexHubFunctions::UnregisterUserCommand(del->usercommandId);
		delete del;
		del = save;
	}
}

void HEXHUBAPI onCmd(long cid, long uid, void *r, void *cp, bool b) {
	char botnick[MAX_NICK];
	char usernick[MAX_NICK];
	std::string out;
	TriggerList *cur = first;
	while (cur != 0) {
		if (cur->triggerId == cid) {
			if (HexHubFunctions::ClientNickFromId(HUBBOT_ID, botnick) == false) return;
			if (b == true) {
				if (HexHubFunctions::ClientNickFromId(uid, usernick) == false) return;
				out = "$To: ";
				out += usernick;
				out += " From: ";
				out += botnick;
				out += " $<";
			} else out = "<";

			out += botnick;
			out += "> ";
			out += cur->text;
			out += "|";
			HexHubFunctions::SendTo(uid, out.c_str(), static_cast<int>(out.length()));
			break;
		}
		cur = cur->next;
	}
}